/*******************************************************************************
 * FileName: Db.js
 * Description: It contains functions used against the local database
 * Copyright (c) 2013 Daniel Franco Cecilia
 *******************************************************************************/

this.Db = function () {
	// Opening database
	var database = window.openDatabase("db_stori", "1.0", "Stori DB", 200000);

	/**
	 * Inserts user into the database
	 * 
	 * @param email. User's email
	 * @param password. User's password
	 */
	this.insertUser = function (email, password) {
		database.transaction(function(tx) {
			tx.executeSql('CREATE TABLE IF NOT EXISTS USER (email, password)');
			tx.executeSql('INSERT INTO USER (email, password) VALUES (?, ?)',[email, password]);			
		}, function (err) {
			console.log("Se ha producido un error en la inserción del usuario: " + err);
		}, function () {
			console.log("Se ha insertado correctamente el usuario con email: " + email + " y password: " + password);
		});	
	};

	/**
	 * Gets user
	 * 
	 * @param callback. Function called after this function ends.
	 */
	this.getUser = function (callback) {
		var user = {email: '', password: ''};

		database.transaction(function(tx) {
			tx.executeSql('CREATE TABLE IF NOT EXISTS USER (email, password)');
			tx.executeSql('SELECT * FROM USER', [], function (tx, results) {
				var len = results.rows.length;
				
				// Checks it's just an user
				if (len === 1) {
					user.email = results.rows.item(0).email;
					user.password = results.rows.item(0).password;
				}

				callback(user);
			});
		});
	};

	/**
	 * Insert a story into the database
	 * 
	 * @param story. {Object} Story to be saved.
	 */
	this.insertStory = function (story) {
		database.transaction(function(tx) {
			tx.executeSql('CREATE TABLE IF NOT EXISTS STORY (title, genre, type, text)');
			tx.executeSql('INSERT INTO STORY (title, genre, type, text) VALUES  (?, ?, ?, ?)', 
					[story.title, story.genre, story.type, story.text]);
		}, function (err) {
			console.log("Se ha producido un error en la inserción de la historia: " + err);
		}, function () {
			console.log("Se ha insertado correctamente la historia: " + story.title);
		});
	};
			
	/**
	 * Update a story
	 * 
	 * @param st. {Object} Story to be saved.
	 * @param callback. Function to be called after the execution.
	 */
	this.updateStory = function (st, callback) {
		database.transaction(function(tx) {
			tx.executeSql('UPDATE STORY SET genre=?, type=?, text=? WHERE title=?',[st.genre, st.type, st.story, st.title]);
		}, function (err) {
			callback(0);
		}, function () {
			callback(1);
		});
	};
	
	/**
	 * Delete a story
	 * 
	 * @param title. {String} title of the story.
	 * @param callback. Function to be called after the execution.
	 */
	this.deleteStory = function (title, callback) {
		database.transaction(function(tx) {
			tx.executeSql('DELETE FROM STORY WHERE title=?',[title]);
		}, function (err) {
			callback(0);
		}, function () {
			callback(1);
		});
	};

	/**
	 * Get a story filtering by title
	 * 
	 * @param title. Story title.
	 */
	this.getStory = function (title, callback) {
		var s = {title: '', genre: '', type: '', text: ''};

		database.transaction(function(tx) {			
			tx.executeSql('SELECT * FROM STORY WHERE title = ?', [title], function (tx, results) {
				var len = results.rows.length;

				if (len === 1) {
					s.title = results.rows.item(0).title;
					s.genre = results.rows.item(0).genre;
					s.type = results.rows.item(0).type;
					s.text = results.rows.item(0).text;
				}

				callback(s);
			});
		});
	};
	

	/**
	 * Get all the stories
	 * 
	 * @param callback. Function called after this function ends.
	 */
	this.getStories = function (callback) {
		var st = {title: '', genre: '', type: '', text: ''};
		var story_array = new Array();

		database.transaction(function(tx) {			
			tx.executeSql('SELECT * FROM STORY', [], function (tx, results) {
				for (var i = 0; i < results.rows.length; i++) {
					st.title = results.rows.item(i).title;
					st.genre = results.rows.item(i).genre;
					st.type = results.rows.item(i).type;
					st.text = results.rows.item(i).text;
			        story_array.push(st);
					st = {title: '', genre: '', type: '', text: ''};
				}
				
				callback(story_array);
			});
		});
	}

	/**
	 * Drops the USER table
	 */
	this.dropTable = function () {
		database.transaction(function(tx) {
			tx.executeSql('DROP TABLE USER', [], function(tx) {}, function(err) {});
		});
	};
}