/*******************************************************************************
 * FileName: index.js
 * Description: It contains main app functionality
 * Copyright (c) 2013 Daniel Franco Cecilia
 *******************************************************************************/

var socket, db, app_user, story = {type: "", genre: "", contacts: "", title: ""}, 
	turn, story_turn = 0, words = 0, room, party = new Array(), 
	refused = new Array(), guests = new Array(), story_card_title = "", turns = new Array();

$(document).on('pageinit', '#home_page',  function() {
	app_user = {email: "laura@gmail.com", password: "12345"}			
	socket = io.connect("http://localhost/", {port: 8080});
	setEventHandlers();	
});

/**
 * Manages the events mappings
 */
var setEventHandlers = function() {
	socket.on("connect", onSocketConnected);
	socket.on("disconnect", onSocketDisconnect);
	socket.on("authenticateUser", onAuthenticateUser);
	socket.on("roomCreated", onRoomCreated);
	socket.on("getInvitation", onGetInvitation);
	socket.on("getMessage", onGetMessage);
	socket.on("getTurn", onGetTurn);
	socket.on("invitationAccepted", onGetInvitationAccepted);
	socket.on("invitationRefused", onGetInvitationRefused);
	socket.on("getStoryWriters", onGetStoryWriters);
	socket.on("getStoryMessage", onGetStoryMessage);
};

/**
 * Socket connected. Sends authentication
 */
function onSocketConnected() {
	console.log("Connected to socket server");				
	socket.emit("authenticateUser", {email: app_user.email, password: app_user.password});
};

/**
 * Socket disconnected
 */
function onSocketDisconnect() {
	console.log("Disconnected from socket server");
};

/**
 * Authentication response
 */
function onAuthenticateUser(data) {
	console.log("Authentication: " + data["response"]);
}

/**
 * Sets the user turn
 * @param data. Data received
 */
function onGetTurn(data) {
	turn = data["turn"];
}

/**
 * Gets an invitation
 * @param data. Data received
 */
function onGetInvitation(data) {
	var r = confirm("The user " + data["user"] + 
			" invited you to create a " + data["type"] + " story " + " of " + data["genre"] + 
			" Do you accept the invitation?");

	// Accepts invitation
	if (r == true) {
		socket.emit("acceptInvitation", {host: data["user"], guest: app_user.email});
		story.type = data["type"];
		story.genre = data["genre"];
		room = data["room"];				
	} else {
		socket.emit("refuseInvitation", {host: data["user"], guest: app_user.email});
	}
}

/**
 * When the user gets an invitation
 * @param data. Data received
 */
function onGetInvitationAccepted(data) {
	console.log("El usuario " + data["guest"] + " ha aceptado tu invitacion");
	party.push(data["guest"]);
	turns.push({user: data["guest"], turn: data["turn"]});
	console.log(party);
}

/**
 * When the user refuses an invitation
 * @param data. Data received
 */
function onGetInvitationRefused(data) {
	console.log("El usuario " + data["guest"] + " ha rechazado tu invitacion");
	refused.push(data["guest"]);	
	console.log(refused);
}

/**
 * Get Message
 * @param data. Data received
 */
function onGetMessage(data) {
	//alert("data:" + data["message"]);
	console.log("message: " + data["message"]);
}

/**
 * Room created. Save the room and the turn.
 * @param data. Data received
 */
function onRoomCreated(data) {
	room = data["room"];
	turn = 1;
	console.log("Se ha creado la room " + room);
}

/**
 * Gets the users whom going to write the story
 * @param data
 */
function onGetStoryWriters(data) {
	var redireccionar = 0;
	story_turn = 1;
	
	if (story.contacts.length === 0) {
		story.contacts = data["users"];
		redireccionar = 1;
	}
	
	if (turns.length === 0) {
		turns = data["turns"];
		redireccionar = 1;
	}
	
	if (redireccionar === 1) {
		$.mobile.changePage( $("#story_creation_page"), "", true, true);
	}
}

/**
 * Gets a story message
 * @param data
 */
function onGetStoryMessage(data) {				
	var message = data["message"], count = 0;	
	story_turn = data["turn"];
	
	if (message !== "") {
		$("#txt_story").val(message);
	}
	
	$("#list_writers").find("label").removeClass("myTurn");
	
	// It's my turn, i underline my user
	if (story_turn == turn) {		
		$("#list_writers").find("label:contains('" + app_user.email + "')").addClass("myTurn");		
	} else {
		for (var i = turns.length; i--;) {
			if (turns[i].turn === story_turn) {
				$("#list_writers").find("label:contains('" + turns[i].user + "')").addClass("myTurn");
				break;
			}
		}
	}
	
	// Updating words left to finish the story
	count = words - $("#txt_story").val().match(/\S+/g).length;
	$("#words").text(count);
}