/*******************************************************************************
 * FileName: server.js
 * Description: Creates the server and attends the requests
 * Copyright (c) 2013 Daniel Franco Cecilia
 *******************************************************************************/

var util = require("util");
var router = require("./router");
var app = require('express')(),
    server = require('http').createServer(app),
    io = require('socket.io').listen(server),
    port = 8080,
    url  = 'http://localhost:' + port + '/';


var socket, clients, handler, router;

/**
 * Initializes the server
 * 
 * @param handlerManager. Handler with the 
 * functions of the server
 */
function init(handlerManager) {
	if (process.env.SUBDOMAIN) {
  		url = 'http://' + process.env.SUBDOMAIN + '.jit.su/';
	}
	
	server.listen(port);
	app.get('/', function (req, res) {
		res.writeHead(200, {
			'Content-type':'text/javascript',
			'Access-Control-Allow-Origin': '*'
		});
	});

	handler = handlerManager;
	setEventHandlers();	
}

/**
 * Listen to the connection event
 */
var setEventHandlers = function() {	
	io.sockets.on("connection", onSocketConnection);	
};

/**
 * Listen to the events
 * 
 * @param client. Client connected
 */
function onSocketConnection(client) {	
	util.log("New client connected: " + client.id);
	
	client.on("authenticateUser", onAuthenticateUser);
	client.on("disconnect", onClientDisconnect);
	client.on("createStory", onCreateStory);
	client.on("startStory", onStartStory);
	client.on("acceptInvitation", onAcceptInvitation);
	client.on("refuseInvitation", onRefuseInvitation);
	client.on("sendStoryMessage", onSendStoryMessage);
}

/**
 * User Authentication. Route to the handler function.
 * @param data
 */
function onAuthenticateUser(data) {
	util.log("Autentificación del usuario");
	router.route(handler, "authenticateUser", io, this, data);
}

/**
 * Client disconnects. Route to the handler function.
 * @param data
 */
function onClientDisconnect(data) {
	util.log("Desconexión del usuario");
	router.route(handler, "disconnect", io, this, data);
}

/**
 * Create a story. Route to the handler function.
 * @param data
 */
function onCreateStory(data) {
	util.log("Invitación a usuarios para crear una historia");
	router.route(handler, "createStory", io, this, data);
}

/**
 * Start a story. Route to the handler function.
 * @param data
 */
function onStartStory(data) {
	util.log("Se va a empezar a escribir una historia");
	router.route(handler, "startStory", io, this, data);
}

/**
 * Accepts an invitation. Route to the handler function.
 * @param data
 */
function onAcceptInvitation(data) {
	util.log("El usuario " + data["guest"] + " ha aceptado la invitación para crear una historia del usuario " + data["host"]);
	router.route(handler, "acceptInvitation", io, this, data);
}

/**
 * Refuses invitation. Route to the handler function.
 * @param data
 */
function onRefuseInvitation(data) {
	util.log("El usuario " + data["guest"] + " ha rechazado la invitación para crear una historia del usuario " + data["host"]);
	router.route(handler, "refuseInvitation", io, this, data);
}

/**
 * Sends story message. Route to the handler function.
 * @param data
 */
function onSendStoryMessage(data) {
	util.log("Enviado nuevo texto");
	router.route(handler, "sendStoryMessage", io, this, data);
}

exports.init = init;